@@ -9,10 +9,11 @@
  * manager to run.
  */
 
-#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
 
+static GtkWidget *window = NULL;
+
 void
 copy_button_clicked (GtkWidget *button,
                      gpointer   user_data)
@@ -67,24 +68,17 @@ paste_button_clicked (GtkWidget *button,
 static GdkPixbuf *
 get_image_pixbuf (GtkImage *image)
 {
-  const gchar *icon_name;
+  gchar *stock_id;
   GtkIconSize size;
-  GtkIconTheme *icon_theme;
-  int width;
 
   switch (gtk_image_get_storage_type (image))
     {
     case GTK_IMAGE_PIXBUF:
       return g_object_ref (gtk_image_get_pixbuf (image));
-    case GTK_IMAGE_ICON_NAME:
-      gtk_image_get_icon_name (image, &icon_name, &size);
-      icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (image)));
-      gtk_icon_size_lookup (size, &width, NULL);
-      return gtk_icon_theme_load_icon (icon_theme,
-                                       icon_name,
-                                       width,
-                                       GTK_ICON_LOOKUP_GENERIC_FALLBACK,
-                                       NULL);
+    case GTK_IMAGE_STOCK:
+      gtk_image_get_stock (image, &stock_id, &size);
+      return gtk_widget_render_icon_pixbuf (GTK_WIDGET (image),
+                                            stock_id, size);
     default:
       g_warning ("Image storage type %d not handled",
                  gtk_image_get_storage_type (image));
@@ -105,12 +99,12 @@ drag_begin (GtkWidget      *widget,
 }
 
 void
-drag_data_get (GtkWidget        *widget,
-               GdkDragContext   *context,
-               GtkSelectionData *selection_data,
-               guint             info,
-               guint             time,
-               gpointer          data)
+drag_data_get  (GtkWidget        *widget,
+                GdkDragContext   *context,
+                GtkSelectionData *selection_data,
+                guint             info,
+                guint             time,
+                gpointer          data)
 {
   GdkPixbuf *pixbuf;
 
@@ -183,12 +177,12 @@ button_press (GtkWidget      *widget,
 
   menu = gtk_menu_new ();
 
-  item = gtk_menu_item_new_with_mnemonic (_("_Copy"));
+  item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
   g_signal_connect (item, "activate", G_CALLBACK (copy_image), data);
   gtk_widget_show (item);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-  item = gtk_menu_item_new_with_mnemonic (_("_Paste"));
+  item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PASTE, NULL);
   g_signal_connect (item, "activate", G_CALLBACK (paste_image), data);
   gtk_widget_show (item);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -200,8 +194,6 @@ button_press (GtkWidget      *widget,
 GtkWidget *
 do_clipboard (GtkWidget *do_widget)
 {
-  static GtkWidget *window = NULL;
-
   if (!window)
     {
       GtkWidget *vbox, *hbox;
@@ -213,7 +205,7 @@ do_clipboard (GtkWidget *do_widget)
       window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
       gtk_window_set_screen (GTK_WINDOW (window),
                              gtk_widget_get_screen (do_widget));
-      gtk_window_set_title (GTK_WINDOW (window), "Clipboard");
+      gtk_window_set_title (GTK_WINDOW (window), "Clipboard demo");
 
       g_signal_connect (window, "destroy",
                         G_CALLBACK (gtk_widget_destroyed), &window);
@@ -236,7 +228,7 @@ do_clipboard (GtkWidget *do_widget)
       gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 
       /* Create the button */
-      button = gtk_button_new_with_mnemonic (_("_Copy"));
+      button = gtk_button_new_from_stock (GTK_STOCK_COPY);
       gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
       g_signal_connect (button, "clicked",
                         G_CALLBACK (copy_button_clicked), entry);
@@ -253,7 +245,7 @@ do_clipboard (GtkWidget *do_widget)
       gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 
       /* Create the button */
-      button = gtk_button_new_with_mnemonic (_("_Paste"));
+      button = gtk_button_new_from_stock (GTK_STOCK_PASTE);
       gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
       g_signal_connect (button, "clicked",
                         G_CALLBACK (paste_button_clicked), entry);
@@ -266,8 +258,8 @@ do_clipboard (GtkWidget *do_widget)
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
       /* Create the first image */
-      image = gtk_image_new_from_icon_name ("dialog-warning",
-                                            GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
+                                        GTK_ICON_SIZE_BUTTON);
       ebox = gtk_event_box_new ();
       gtk_container_add (GTK_CONTAINER (ebox), image);
       gtk_container_add (GTK_CONTAINER (hbox), ebox);
@@ -292,8 +284,8 @@ do_clipboard (GtkWidget *do_widget)
                         G_CALLBACK (button_press), image);
 
       /* Create the second image */
-      image = gtk_image_new_from_icon_name ("process-stop",
-                                            GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_stock (GTK_STOCK_STOP,
+                                        GTK_ICON_SIZE_BUTTON);
       ebox = gtk_event_box_new ();
       gtk_container_add (GTK_CONTAINER (ebox), image);
       gtk_container_add (GTK_CONTAINER (hbox), ebox);
@@ -325,7 +317,10 @@ do_clipboard (GtkWidget *do_widget)
   if (!gtk_widget_get_visible (window))
     gtk_widget_show_all (window);
   else
-    gtk_widget_destroy (window);
+    {
+      gtk_widget_destroy (window);
+      window = NULL;
+    }
 
   return window;
 }
