@@ -4,7 +4,6 @@
 #include <glib.h>
 #include <gio/gio.h>
 #include "broadway-protocol.h"
-#include "broadway-buffer.h"
 
 typedef struct BroadwayOutput BroadwayOutput;
 
@@ -18,12 +17,12 @@ typedef enum {
 } BroadwayWSOpCode;
 
 BroadwayOutput *broadway_output_new             (GOutputStream  *out,
-						 guint32         serial);
+						 guint32         serial,
+						 gboolean        proto_v7_plus,
+						 gboolean        binary);
 void            broadway_output_free            (BroadwayOutput *output);
 int             broadway_output_flush           (BroadwayOutput *output);
 int             broadway_output_has_error       (BroadwayOutput *output);
-void            broadway_output_set_next_serial (BroadwayOutput *output,
-						 guint32         serial);
 guint32         broadway_output_get_next_serial (BroadwayOutput *output);
 void            broadway_output_new_surface     (BroadwayOutput *output,
 						 int             id,
@@ -32,15 +31,10 @@ void            broadway_output_new_surface     (BroadwayOutput *output,
 						 int             w,
 						 int             h,
 						 gboolean        is_temp);
-void            broadway_output_disconnected    (BroadwayOutput *output);
 void            broadway_output_show_surface    (BroadwayOutput *output,
 						 int             id);
 void            broadway_output_hide_surface    (BroadwayOutput *output,
 						 int             id);
-void            broadway_output_raise_surface   (BroadwayOutput *output,
-                                                 int             id);
-void            broadway_output_lower_surface   (BroadwayOutput *output,
-                                                 int             id);
 void            broadway_output_destroy_surface (BroadwayOutput *output,
 						 int             id);
 void            broadway_output_move_resize_surface (BroadwayOutput *output,
@@ -54,16 +48,34 @@ void            broadway_output_move_resize_surface (BroadwayOutput *output,
 void            broadway_output_set_transient_for (BroadwayOutput *output,
 						   int             id,
 						   int             parent_id);
-void            broadway_output_put_buffer      (BroadwayOutput *output,
+void            broadway_output_put_rgb         (BroadwayOutput *output,
 						 int             id,
-                                                 BroadwayBuffer *prev_buffer,
-                                                 BroadwayBuffer *buffer);
+						 int             x,
+						 int             y,
+						 int             w,
+						 int             h,
+						 int             byte_stride,
+						 void           *data);
+void            broadway_output_put_rgba        (BroadwayOutput *output,
+						 int             id,
+						 int             x,
+						 int             y,
+						 int             w,
+						 int             h,
+						 int             byte_stride,
+						 void           *data);
+void            broadway_output_surface_flush   (BroadwayOutput *output,
+						 int             id);
+void            broadway_output_copy_rectangles (BroadwayOutput *output,
+						 int             id,
+						 BroadwayRect   *rects,
+						 int             n_rects,
+						 int             dx,
+						 int             dy);
 void            broadway_output_grab_pointer    (BroadwayOutput *output,
 						 int id,
 						 gboolean owner_event);
 guint32         broadway_output_ungrab_pointer  (BroadwayOutput *output);
 void            broadway_output_pong            (BroadwayOutput *output);
-void            broadway_output_set_show_keyboard (BroadwayOutput *output,
-                                                   gboolean show);
 
 #endif /* __BROADWAY_H__ */
