@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -26,10 +28,9 @@
 
 #include "gdkframeclockprivate.h"
 #include "gdkinternals.h"
-#include "gdkprofilerprivate.h"
 
 /**
- * SECTION:gdkframeclock
+ * SECTION:frameclock
  * @Short_description: Frame clock syncs painting to a window or display
  * @Title: Frame clock
  *
@@ -51,23 +52,25 @@
  * sense for the synchronization being implemented, the clock will
  * process a frame and emit signals for each phase that has been
  * requested. (See the signals of the #GdkFrameClock class for
- * documentation of the phases. %GDK_FRAME_CLOCK_PHASE_UPDATE and the
- * #GdkFrameClock::update signal are most interesting for application
- * writers, and are used to update the animations, using the frame time
- * given by gdk_frame_clock_get_frame_time().
+ * documentation of the phases. GDK_FRAME_CLOCK_PHASE_UPDATE and the
+ * ::update signal are most interesting for application writers, and
+ * are used to update the animations, using the frame time given by
+ * gdk_frame_clock_get_frame_time().
  *
  * The frame time is reported in microseconds and generally in the same
  * timescale as g_get_monotonic_time(), however, it is not the same
  * as g_get_monotonic_time(). The frame time does not advance during
  * the time a frame is being painted, and outside of a frame, an attempt
  * is made so that all calls to gdk_frame_clock_get_frame_time() that
- * are called at a “similar” time get the same value. This means that
+ * are called at a "similar" time get the same value. This means that
  * if different animations are timed by looking at the difference in
  * time between an initial value from gdk_frame_clock_get_frame_time()
- * and the value inside the #GdkFrameClock::update signal of the clock,
- * they will stay exactly synchronized.
+ * and the value inside the ::update signal of the clock, they will
+ * stay exactly synchronized.
  */
 
+G_DEFINE_ABSTRACT_TYPE (GdkFrameClock, gdk_frame_clock, G_TYPE_OBJECT)
+
 enum {
   FLUSH_EVENTS,
   BEFORE_PAINT,
@@ -81,10 +84,6 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 
-#ifdef G_ENABLE_DEBUG
-static guint fps_counter;
-#endif
-
 #define FRAME_HISTORY_MAX_LENGTH 16
 
 struct _GdkFrameClockPrivate
@@ -95,8 +94,6 @@ struct _GdkFrameClockPrivate
   GdkFrameTimings *timings[FRAME_HISTORY_MAX_LENGTH];
 };
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GdkFrameClock, gdk_frame_clock, G_TYPE_OBJECT)
-
 static void
 gdk_frame_clock_finalize (GObject *object)
 {
@@ -130,7 +127,8 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
                   GDK_TYPE_FRAME_CLOCK,
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -145,7 +143,8 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
                   GDK_TYPE_FRAME_CLOCK,
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -164,7 +163,8 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
                   GDK_TYPE_FRAME_CLOCK,
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -174,14 +174,15 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
    * This signal is emitted as the second step of toolkit and
    * application processing of the frame. Any work to update
    * sizes and positions of application elements should be
-   * performed. GTK+ normally handles this internally.
+   * performed. GTK normally handles this internally.
    */
   signals[LAYOUT] =
     g_signal_new (g_intern_static_string ("layout"),
                   GDK_TYPE_FRAME_CLOCK,
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -191,15 +192,16 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
    * This signal is emitted as the third step of toolkit and
    * application processing of the frame. The frame is
    * repainted. GDK normally handles this internally and
-   * produces expose events, which are turned into GTK+
-   * #GtkWidget::draw signals.
+   * produce expose events, which are turned into GTK
+   * GtkWidget::draw signals.
    */
   signals[PAINT] =
     g_signal_new (g_intern_static_string ("paint"),
                   GDK_TYPE_FRAME_CLOCK,
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -214,7 +216,8 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
                   GDK_TYPE_FRAME_CLOCK,
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -230,8 +233,11 @@ gdk_frame_clock_class_init (GdkFrameClockClass *klass)
                   GDK_TYPE_FRAME_CLOCK,
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
+
+  g_type_class_add_private (klass, sizeof (GdkFrameClockPrivate));
 }
 
 static void
@@ -239,15 +245,13 @@ gdk_frame_clock_init (GdkFrameClock *clock)
 {
   GdkFrameClockPrivate *priv;
 
-  clock->priv = priv = gdk_frame_clock_get_instance_private (clock);
+  clock->priv = G_TYPE_INSTANCE_GET_PRIVATE (clock,
+                                             GDK_TYPE_FRAME_CLOCK,
+                                             GdkFrameClockPrivate);
+  priv = clock->priv;
 
   priv->frame_counter = -1;
   priv->current = FRAME_HISTORY_MAX_LENGTH - 1;
-
-#ifdef G_ENABLE_DEBUG
-  if (fps_counter == 0)
-    fps_counter = gdk_profiler_define_counter ("fps", "Frames per Second");
-#endif
 }
 
 /**
@@ -255,14 +259,14 @@ gdk_frame_clock_init (GdkFrameClock *clock)
  * @frame_clock: a #GdkFrameClock
  *
  * Gets the time that should currently be used for animations.  Inside
- * the processing of a frame, it’s the time used to compute the
+ * the processing of a frame, it's the time used to compute the
  * animation position of everything in a frame. Outside of a frame, it's
- * the time of the conceptual “previous frame,” which may be either
- * the actual previous frame time, or if that’s too old, an updated
+ * the time of the conceptual "previous frame," which may be either
+ * the actual previous frame time, or if that's too old, an updated
  * time.
  *
  * Since: 3.8
- * Returns: a timestamp in microseconds, in the timescale of
+ * Return value: a timestamp in microseconds, in the timescale of
  *  of g_get_monotonic_time().
  */
 gint64
@@ -276,12 +280,11 @@ gdk_frame_clock_get_frame_time (GdkFrameClock *frame_clock)
 /**
  * gdk_frame_clock_request_phase:
  * @frame_clock: a #GdkFrameClock
- * @phase: the phase that is requested
  *
  * Asks the frame clock to run a particular phase. The signal
  * corresponding the requested phase will be emitted the next
  * time the frame clock processes. Multiple calls to
- * gdk_frame_clock_request_phase() will be combined together
+ * gdk_frame_clock_request_phase() will be combined togethe
  * and only one frame processed. If you are displaying animated
  * content and want to continually request the
  * %GDK_FRAME_CLOCK_PHASE_UPDATE phase for a period of time,
@@ -383,14 +386,14 @@ gdk_frame_clock_get_frame_counter (GdkFrameClock *frame_clock)
  * gdk_frame_clock_get_history_start:
  * @frame_clock: a #GdkFrameClock
  *
- * #GdkFrameClock internally keeps a history of #GdkFrameTimings
+ * #GdkFrameClock internally keeps a history of #GdkFrameTiming
  * objects for recent frames that can be retrieved with
  * gdk_frame_clock_get_timings(). The set of stored frames
  * is the set from the counter values given by
  * gdk_frame_clock_get_history_start() and
  * gdk_frame_clock_get_frame_counter(), inclusive.
  *
- * Returns: the frame counter value for the oldest frame
+ * Return value: the frame counter value for the oldest frame
  *  that is available in the internal frame history of the
  *  #GdkFrameClock.
  * Since: 3.8
@@ -419,18 +422,12 @@ _gdk_frame_clock_begin_frame (GdkFrameClock *frame_clock)
   priv->frame_counter++;
   priv->current = (priv->current + 1) % FRAME_HISTORY_MAX_LENGTH;
 
-  /* Try to steal the previous frame timing instead of discarding
-   * and allocating a new one.
-   */
-  if G_LIKELY (priv->n_timings == FRAME_HISTORY_MAX_LENGTH &&
-               _gdk_frame_timings_steal (priv->timings[priv->current],
-                                         priv->frame_counter))
-    return;
-
   if (priv->n_timings < FRAME_HISTORY_MAX_LENGTH)
     priv->n_timings++;
   else
-    gdk_frame_timings_unref (priv->timings[priv->current]);
+    {
+      gdk_frame_timings_unref(priv->timings[priv->current]);
+    }
 
   priv->timings[priv->current] = _gdk_frame_timings_new (priv->frame_counter);
 }
@@ -445,8 +442,8 @@ _gdk_frame_clock_begin_frame (GdkFrameClock *frame_clock)
  * for the current frame or a recent frame. The #GdkFrameTimings
  * object may not yet be complete: see gdk_frame_timings_get_complete().
  *
- * Returns: (nullable) (transfer none): the #GdkFrameTimings object for
- *  the specified frame, or %NULL if it is not available. See
+ * Return value: the #GdkFrameTimings object for the specified
+ *  frame, or %NULL if it is not available. See
  *  gdk_frame_clock_get_history_start().
  * Since: 3.8
  */
@@ -478,10 +475,10 @@ gdk_frame_clock_get_timings (GdkFrameClock *frame_clock,
  *
  * Gets the frame timings for the current frame.
  *
- * Returns: (nullable) (transfer none): the #GdkFrameTimings for the
- *  frame currently being processed, or even no frame is being
- *  processed, for the previous frame. Before any frames have been
- *  processed, returns %NULL.
+ * Returns: the #GdkFrameTimings for the frame currently being
+ *  processed, or even no frame is being processed, for the
+ *  previous frame. Before any frames have been procesed,
+ *  returns %NULL.
  * Since: 3.8
  */
 GdkFrameTimings *
@@ -502,8 +499,6 @@ void
 _gdk_frame_clock_debug_print_timings (GdkFrameClock   *clock,
                                       GdkFrameTimings *timings)
 {
-  GString *str;
-
   gint64 previous_frame_time = 0;
   GdkFrameTimings *previous_timings = gdk_frame_clock_get_timings (clock,
                                                                    timings->frame_counter - 1);
@@ -511,29 +506,25 @@ _gdk_frame_clock_debug_print_timings (GdkFrameClock   *clock,
   if (previous_timings != NULL)
     previous_frame_time = previous_timings->frame_time;
 
-  str = g_string_new ("");
-
-  g_string_append_printf (str, "%5" G_GINT64_FORMAT ":", timings->frame_counter);
+  g_print ("%5" G_GINT64_FORMAT ":", timings->frame_counter);
   if (previous_frame_time != 0)
     {
-      g_string_append_printf (str, " interval=%-4.1f", (timings->frame_time - previous_frame_time) / 1000.);
-      g_string_append_printf (str, timings->slept_before ?  " (sleep)" : "        ");
+      g_print (" interval=%-4.1f", (timings->frame_time - previous_frame_time) / 1000.);
+      g_print (timings->slept_before ?  " (sleep)" : "        ");
     }
   if (timings->layout_start_time != 0)
-    g_string_append_printf (str, " layout_start=%-4.1f", (timings->layout_start_time - timings->frame_time) / 1000.);
+    g_print (" layout_start=%-4.1f", (timings->layout_start_time - timings->frame_time) / 1000.);
   if (timings->paint_start_time != 0)
-    g_string_append_printf (str, " paint_start=%-4.1f", (timings->paint_start_time - timings->frame_time) / 1000.);
+    g_print (" paint_start=%-4.1f", (timings->paint_start_time - timings->frame_time) / 1000.);
   if (timings->frame_end_time != 0)
-    g_string_append_printf (str, " frame_end=%-4.1f", (timings->frame_end_time - timings->frame_time) / 1000.);
+    g_print (" frame_end=%-4.1f", (timings->frame_end_time - timings->frame_time) / 1000.);
   if (timings->presentation_time != 0)
-    g_string_append_printf (str, " present=%-4.1f", (timings->presentation_time - timings->frame_time) / 1000.);
+    g_print (" present=%-4.1f", (timings->presentation_time - timings->frame_time) / 1000.);
   if (timings->predicted_presentation_time != 0)
-    g_string_append_printf (str, " predicted=%-4.1f", (timings->predicted_presentation_time - timings->frame_time) / 1000.);
+    g_print (" predicted=%-4.1f", (timings->predicted_presentation_time - timings->frame_time) / 1000.);
   if (timings->refresh_interval != 0)
-    g_string_append_printf (str, " refresh_interval=%-4.1f", timings->refresh_interval / 1000.);
-
-  g_message ("%s", str->str);
-  g_string_free (str, TRUE);
+    g_print (" refresh_interval=%-4.1f", timings->refresh_interval / 1000.);
+  g_print ("\n");
 }
 #endif /* G_ENABLE_DEBUG */
 
@@ -544,10 +535,10 @@ _gdk_frame_clock_debug_print_timings (GdkFrameClock   *clock,
  * gdk_frame_clock_get_refresh_info:
  * @frame_clock: a #GdkFrameClock
  * @base_time: base time for determining a presentaton time
- * @refresh_interval_return: (out) (optional): a location to store the
- * determined refresh interval, or %NULL. A default refresh interval of
- * 1/60th of a second will be stored if no history is present.
- * @presentation_time_return: (out): a location to store the next
+ * @refresh_interval_return: a location to store the determined refresh
+ *  interval, or %NULL. A default refresh interval of 1/60th of
+ *  a second will be stored if no history is present.
+ * @presentation_time_return: a location to store the next
  *  candidate presentation time after the given base time.
  *  0 will be will be stored if no history is present.
  *
@@ -612,147 +603,3 @@ gdk_frame_clock_get_refresh_info (GdkFrameClock *frame_clock,
       frame_counter--;
     }
 }
-
-void
-_gdk_frame_clock_emit_flush_events (GdkFrameClock *frame_clock)
-{
-  g_signal_emit (frame_clock, signals[FLUSH_EVENTS], 0);
-}
-
-void
-_gdk_frame_clock_emit_before_paint (GdkFrameClock *frame_clock)
-{
-  g_signal_emit (frame_clock, signals[BEFORE_PAINT], 0);
-}
-
-void
-_gdk_frame_clock_emit_update (GdkFrameClock *frame_clock)
-{
-  g_signal_emit (frame_clock, signals[UPDATE], 0);
-}
-
-void
-_gdk_frame_clock_emit_layout (GdkFrameClock *frame_clock)
-{
-  g_signal_emit (frame_clock, signals[LAYOUT], 0);
-}
-
-void
-_gdk_frame_clock_emit_paint (GdkFrameClock *frame_clock)
-{
-  g_signal_emit (frame_clock, signals[PAINT], 0);
-}
-
-void
-_gdk_frame_clock_emit_after_paint (GdkFrameClock *frame_clock)
-{
-  g_signal_emit (frame_clock, signals[AFTER_PAINT], 0);
-}
-
-void
-_gdk_frame_clock_emit_resume_events (GdkFrameClock *frame_clock)
-{
-  g_signal_emit (frame_clock, signals[RESUME_EVENTS], 0);
-}
-
-#ifdef G_ENABLE_DEBUG
-static gint64
-guess_refresh_interval (GdkFrameClock *frame_clock)
-{
-  gint64 interval;
-  gint64 i;
-
-  interval = G_MAXINT64;
-
-  for (i = gdk_frame_clock_get_history_start (frame_clock);
-       i < gdk_frame_clock_get_frame_counter (frame_clock);
-       i++)
-    {
-      GdkFrameTimings *t, *before;
-      gint64 ts, before_ts;
-
-      t = gdk_frame_clock_get_timings (frame_clock, i);
-      before = gdk_frame_clock_get_timings (frame_clock, i - 1);
-      if (t == NULL || before == NULL)
-        continue;
-
-      ts = gdk_frame_timings_get_frame_time (t);
-      before_ts = gdk_frame_timings_get_frame_time (before);
-      if (ts == 0 || before_ts == 0)
-        continue;
-
-      interval = MIN (interval, ts - before_ts);
-    }
-
-  if (interval == G_MAXINT64)
-    return 0;
-
-  return interval;
-}
-
-static double
-frame_clock_get_fps (GdkFrameClock *frame_clock)
-{
-  GdkFrameTimings *start, *end;
-  gint64 start_counter, end_counter;
-  gint64 start_timestamp, end_timestamp;
-  gint64 interval;
-
-  start_counter = gdk_frame_clock_get_history_start (frame_clock);
-  end_counter = gdk_frame_clock_get_frame_counter (frame_clock);
-  start = gdk_frame_clock_get_timings (frame_clock, start_counter);
-  for (end = gdk_frame_clock_get_timings (frame_clock, end_counter);
-       end_counter > start_counter && end != NULL && !gdk_frame_timings_get_complete (end);
-       end = gdk_frame_clock_get_timings (frame_clock, end_counter))
-    end_counter--;
-  if (end_counter - start_counter < 4)
-    return 0.0;
-
-  start_timestamp = gdk_frame_timings_get_presentation_time (start);
-  end_timestamp = gdk_frame_timings_get_presentation_time (end);
-  if (start_timestamp == 0 || end_timestamp == 0)
-    {
-      start_timestamp = gdk_frame_timings_get_frame_time (start);
-      end_timestamp = gdk_frame_timings_get_frame_time (end);
-    }
-  interval = gdk_frame_timings_get_refresh_interval (end);
-  if (interval == 0)
-    {
-      interval = guess_refresh_interval (frame_clock);
-      if (interval == 0)
-        return 0.0;
-    }   
-    
-  return ((double) end_counter - start_counter) * G_USEC_PER_SEC / (end_timestamp - start_timestamp);
-}
-#endif
-
-void
-_gdk_frame_clock_add_timings_to_profiler (GdkFrameClock   *clock,
-                                          GdkFrameTimings *timings)
-{
-#ifdef G_ENABLE_DEBUG
-  gdk_profiler_add_mark (timings->frame_time * 1000,
-                         (timings->frame_end_time - timings->frame_time) * 1000,
-                         "frame", "");
-
-  if (timings->layout_start_time != 0)
-    gdk_profiler_add_mark (timings->layout_start_time * 1000,
-                           (timings->paint_start_time - timings->layout_start_time) * 1000,
-                            "layout", "");
-
-  if (timings->paint_start_time != 0)
-    gdk_profiler_add_mark (timings->paint_start_time * 1000,
-                           (timings->frame_end_time - timings->paint_start_time) * 1000,
-                            "paint", "");
-
-  if (timings->presentation_time != 0)
-    gdk_profiler_add_mark (timings->presentation_time * 1000,
-                           0,
-                           "presentation", "");
-
-  gdk_profiler_set_counter (fps_counter,
-                            timings->frame_end_time * 1000,
-                            frame_clock_get_fps (clock));
-#endif
-}
