@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -27,23 +29,22 @@
 #include "gdkinternals.h"
 #include "gdkframeclockprivate.h"
 #include "gdkframeclockidle.h"
-#include "gdkprofilerprivate.h"
 #include "gdk.h"
 
 #ifdef G_OS_WIN32
 #include <windows.h>
 #endif
 
-#define FRAME_INTERVAL 16667 /* microseconds */
+#define FRAME_INTERVAL 16667 // microseconds
 
 struct _GdkFrameClockIdlePrivate
 {
+  GTimer *timer;
+  /* timer_base is used to avoid ever going backward */
+  gint64 timer_base;
   gint64 frame_time;
   gint64 min_next_frame_time;
   gint64 sleep_serial;
-#ifdef G_ENABLE_DEBUG
-  gint64 freeze_time;
-#endif
 
   guint flush_idle_id;
   guint paint_idle_id;
@@ -62,7 +63,7 @@ struct _GdkFrameClockIdlePrivate
 static gboolean gdk_frame_clock_flush_idle (void *data);
 static gboolean gdk_frame_clock_paint_idle (void *data);
 
-G_DEFINE_TYPE_WITH_PRIVATE (GdkFrameClockIdle, gdk_frame_clock_idle, GDK_TYPE_FRAME_CLOCK)
+G_DEFINE_TYPE (GdkFrameClockIdle, gdk_frame_clock_idle, GDK_TYPE_FRAME_CLOCK)
 
 static gint64 sleep_serial;
 static gint64 sleep_source_prepare_time;
@@ -121,10 +122,11 @@ gdk_frame_clock_idle_init (GdkFrameClockIdle *frame_clock_idle)
 {
   GdkFrameClockIdlePrivate *priv;
 
-  frame_clock_idle->priv = priv =
-    gdk_frame_clock_idle_get_instance_private (frame_clock_idle);
+  frame_clock_idle->priv = G_TYPE_INSTANCE_GET_PRIVATE (frame_clock_idle,
+                                                        GDK_TYPE_FRAME_CLOCK_IDLE,
+                                                        GdkFrameClockIdlePrivate);
+  priv = frame_clock_idle->priv;
 
-  priv->frame_time = g_get_monotonic_time (); /* more sane than zero */
   priv->freeze_count = 0;
 }
 
@@ -161,12 +163,22 @@ compute_frame_time (GdkFrameClockIdle *idle)
 {
   GdkFrameClockIdlePrivate *priv = idle->priv;
   gint64 computed_frame_time;
+  gint64 elapsed;
 
-  computed_frame_time = g_get_monotonic_time ();
-
-  /* ensure monotonicity of frame time */
-  if (computed_frame_time <= priv->frame_time)
+  elapsed = g_get_monotonic_time () + priv->timer_base;
+  if (elapsed < priv->frame_time)
+    {
+      /* clock went backward. adapt to that by forevermore increasing
+       * timer_base.  For now, assume we've gone forward in time 1ms.
+       */
+      /* hmm. just fix GTimer? */
       computed_frame_time = priv->frame_time + 1;
+      priv->timer_base += (priv->frame_time - elapsed) + 1;
+    }
+  else
+    {
+      computed_frame_time = elapsed;
+    }
 
   return computed_frame_time;
 }
@@ -233,18 +245,15 @@ maybe_start_idle (GdkFrameClockIdle *clock_idle)
                                                               gdk_frame_clock_flush_idle,
                                                               g_object_ref (clock_idle),
                                                               (GDestroyNotify) g_object_unref);
-          g_source_set_name_by_id (priv->flush_idle_id, "[gtk+] gdk_frame_clock_flush_idle");
         }
 
-      if (!priv->in_paint_idle &&
-	  priv->paint_idle_id == 0 && RUN_PAINT_IDLE (priv))
+      if (priv->paint_idle_id == 0 && RUN_PAINT_IDLE (priv))
         {
           priv->paint_idle_id = gdk_threads_add_timeout_full (GDK_PRIORITY_REDRAW,
                                                               min_interval,
                                                               gdk_frame_clock_paint_idle,
                                                               g_object_ref (clock_idle),
                                                               (GDestroyNotify) g_object_unref);
-          g_source_set_name_by_id (priv->paint_idle_id, "[gtk+] gdk_frame_clock_paint_idle");
         }
     }
 }
@@ -299,7 +308,7 @@ gdk_frame_clock_flush_idle (void *data)
   priv->phase = GDK_FRAME_CLOCK_PHASE_FLUSH_EVENTS;
   priv->requested &= ~GDK_FRAME_CLOCK_PHASE_FLUSH_EVENTS;
 
-  _gdk_frame_clock_emit_flush_events (clock);
+  g_signal_emit_by_name (G_OBJECT (clock), "flush-events");
 
   if ((priv->requested & ~GDK_FRAME_CLOCK_PHASE_FLUSH_EVENTS) != 0 ||
       priv->updating_count > 0)
@@ -342,37 +351,9 @@ gdk_frame_clock_paint_idle (void *data)
         case GDK_FRAME_CLOCK_PHASE_BEFORE_PAINT:
           if (priv->freeze_count == 0)
             {
-              gint64 frame_interval = FRAME_INTERVAL;
-              gint64 reset_frame_time;
-              gint64 smoothest_frame_time;
-              gint64 frame_time_error;
-              GdkFrameTimings *prev_timings =
-                gdk_frame_clock_get_current_timings (clock);
-
-              if (prev_timings && prev_timings->refresh_interval)
-                frame_interval = prev_timings->refresh_interval;
-
-              /* We are likely not getting precisely even callbacks in real
-               * time, particularly if the event loop is busy.
-               * This is a documented limitation in the precision of
-               * gdk_threads_add_timeout_full and g_timeout_add_full.
-               *
-               * In order to avoid this imprecision from compounding between
-               * frames and affecting visual smoothness, we correct frame_time
-               * to more precisely match the even refresh interval of the
-               * physical display. This also means we proactively avoid (most)
-               * missed frames before they occur.
-               */
-              smoothest_frame_time = priv->frame_time + frame_interval;
-              reset_frame_time = compute_frame_time (clock_idle);
-              frame_time_error = ABS (reset_frame_time - smoothest_frame_time);
-              if (frame_time_error >= frame_interval)
-                priv->frame_time = reset_frame_time;
-              else
-                priv->frame_time = smoothest_frame_time;
+              priv->frame_time = compute_frame_time (clock_idle);
 
               _gdk_frame_clock_begin_frame (clock);
-              /* Note "current" is different now so timings != prev_timings */
               timings = gdk_frame_clock_get_current_timings (clock);
 
               timings->frame_time = priv->frame_time;
@@ -383,13 +364,11 @@ gdk_frame_clock_paint_idle (void *data)
               /* We always emit ::before-paint and ::after-paint if
                * any of the intermediate phases are requested and
                * they don't get repeated if you freeze/thaw while
-               * in them.
-               */
+               * in them. */
               priv->requested &= ~GDK_FRAME_CLOCK_PHASE_BEFORE_PAINT;
-              _gdk_frame_clock_emit_before_paint (clock);
+              g_signal_emit_by_name (G_OBJECT (clock), "before-paint");
               priv->phase = GDK_FRAME_CLOCK_PHASE_UPDATE;
             }
-          /* fallthrough */
         case GDK_FRAME_CLOCK_PHASE_UPDATE:
           if (priv->freeze_count == 0)
             {
@@ -397,16 +376,14 @@ gdk_frame_clock_paint_idle (void *data)
                   priv->updating_count > 0)
                 {
                   priv->requested &= ~GDK_FRAME_CLOCK_PHASE_UPDATE;
-                  _gdk_frame_clock_emit_update (clock);
+                  g_signal_emit_by_name (G_OBJECT (clock), "update");
                 }
             }
-          /* fallthrough */
         case GDK_FRAME_CLOCK_PHASE_LAYOUT:
           if (priv->freeze_count == 0)
             {
-	      int iter;
 #ifdef G_ENABLE_DEBUG
-              if (GDK_DEBUG_CHECK (FRAMES) || gdk_profiler_is_running ())
+              if ((_gdk_debug_flags & GDK_DEBUG_FRAMES) != 0)
                 {
                   if (priv->phase != GDK_FRAME_CLOCK_PHASE_LAYOUT &&
                       (priv->requested & GDK_FRAME_CLOCK_PHASE_LAYOUT))
@@ -415,27 +392,17 @@ gdk_frame_clock_paint_idle (void *data)
 #endif /* G_ENABLE_DEBUG */
 
               priv->phase = GDK_FRAME_CLOCK_PHASE_LAYOUT;
-	      /* We loop in the layout phase, because we don't want to progress
-	       * into the paint phase with invalid size allocations. This may
-	       * happen in some situation like races between user window
-	       * resizes and natural size changes.
-	       */
-	      iter = 0;
-              while ((priv->requested & GDK_FRAME_CLOCK_PHASE_LAYOUT) &&
-		     priv->freeze_count == 0 && iter++ < 4)
+              if (priv->requested & GDK_FRAME_CLOCK_PHASE_LAYOUT)
                 {
                   priv->requested &= ~GDK_FRAME_CLOCK_PHASE_LAYOUT;
-                  _gdk_frame_clock_emit_layout (clock);
+                  g_signal_emit_by_name (G_OBJECT (clock), "layout");
                 }
-	      if (iter == 5)
-		g_warning ("gdk-frame-clock: layout continuously requested, giving up after 4 tries");
             }
-          /* fallthrough */
         case GDK_FRAME_CLOCK_PHASE_PAINT:
           if (priv->freeze_count == 0)
             {
 #ifdef G_ENABLE_DEBUG
-              if (GDK_DEBUG_CHECK (FRAMES) || gdk_profiler_is_running ())
+              if ((_gdk_debug_flags & GDK_DEBUG_FRAMES) != 0)
                 {
                   if (priv->phase != GDK_FRAME_CLOCK_PHASE_PAINT &&
                       (priv->requested & GDK_FRAME_CLOCK_PHASE_PAINT))
@@ -447,34 +414,32 @@ gdk_frame_clock_paint_idle (void *data)
               if (priv->requested & GDK_FRAME_CLOCK_PHASE_PAINT)
                 {
                   priv->requested &= ~GDK_FRAME_CLOCK_PHASE_PAINT;
-                  _gdk_frame_clock_emit_paint (clock);
+                  g_signal_emit_by_name (G_OBJECT (clock), "paint");
                 }
             }
-          /* fallthrough */
         case GDK_FRAME_CLOCK_PHASE_AFTER_PAINT:
           if (priv->freeze_count == 0)
             {
               priv->requested &= ~GDK_FRAME_CLOCK_PHASE_AFTER_PAINT;
-              _gdk_frame_clock_emit_after_paint (clock);
+              g_signal_emit_by_name (G_OBJECT (clock), "after-paint");
               /* the ::after-paint phase doesn't get repeated on freeze/thaw,
                */
               priv->phase = GDK_FRAME_CLOCK_PHASE_NONE;
 
 #ifdef G_ENABLE_DEBUG
-              if (GDK_DEBUG_CHECK (FRAMES) || gdk_profiler_is_running ())
+              if ((_gdk_debug_flags & GDK_DEBUG_FRAMES) != 0)
                 timings->frame_end_time = g_get_monotonic_time ();
 #endif /* G_ENABLE_DEBUG */
             }
-          /* fallthrough */
         case GDK_FRAME_CLOCK_PHASE_RESUME_EVENTS:
           ;
         }
     }
 
 #ifdef G_ENABLE_DEBUG
-  if (GDK_DEBUG_CHECK (FRAMES))
+  if ((_gdk_debug_flags & GDK_DEBUG_FRAMES) != 0)
     {
-      if (timings && timings->complete)
+      if (timings->complete)
         _gdk_frame_clock_debug_print_timings (clock, timings);
     }
 #endif /* G_ENABLE_DEBUG */
@@ -482,7 +447,7 @@ gdk_frame_clock_paint_idle (void *data)
   if (priv->requested & GDK_FRAME_CLOCK_PHASE_RESUME_EVENTS)
     {
       priv->requested &= ~GDK_FRAME_CLOCK_PHASE_RESUME_EVENTS;
-      _gdk_frame_clock_emit_resume_events (clock);
+      g_signal_emit_by_name (G_OBJECT (clock), "resume-events");
     }
 
   if (priv->freeze_count == 0)
@@ -563,14 +528,6 @@ gdk_frame_clock_idle_freeze (GdkFrameClock *clock)
   GdkFrameClockIdle *clock_idle = GDK_FRAME_CLOCK_IDLE (clock);
   GdkFrameClockIdlePrivate *priv = clock_idle->priv;
 
-#ifdef G_ENABLE_DEBUG
-  if (priv->freeze_count == 0)
-    {
-      if (gdk_profiler_is_running ())
-        priv->freeze_time = g_get_monotonic_time ();
-    }
-#endif
-
   priv->freeze_count++;
   maybe_stop_idle (clock_idle);
 }
@@ -589,26 +546,11 @@ gdk_frame_clock_idle_thaw (GdkFrameClock *clock)
       maybe_start_idle (clock_idle);
       /* If nothing is requested so we didn't start an idle, we need
        * to skip to the end of the state chain, since the idle won't
-       * run and do it for us.
-       */
+       * run and do it for us. */
       if (priv->paint_idle_id == 0)
         priv->phase = GDK_FRAME_CLOCK_PHASE_NONE;
 
       priv->sleep_serial = get_sleep_serial ();
-
-#ifdef G_ENABLE_DEBUG
-      if (gdk_profiler_is_running ())
-        {
-          if (priv->freeze_time != 0)
-            {
-              gint64 thaw_time = g_get_monotonic_time ();
-              gdk_profiler_add_mark (priv->freeze_time * 1000,
-                                     (thaw_time - priv->freeze_time) * 1000,
-                                     "freeze", "");
-              priv->freeze_time = 0;
-            }
-        }
-#endif
     }
 }
 
@@ -626,6 +568,8 @@ gdk_frame_clock_idle_class_init (GdkFrameClockIdleClass *klass)
   frame_clock_class->end_updating = gdk_frame_clock_idle_end_updating;
   frame_clock_class->freeze = gdk_frame_clock_idle_freeze;
   frame_clock_class->thaw = gdk_frame_clock_idle_thaw;
+
+  g_type_class_add_private (klass, sizeof (GdkFrameClockIdlePrivate));
 }
 
 GdkFrameClock *
