@@ -276,16 +276,8 @@ _gdk_x11_precache_atoms (GdkDisplay          *display,
     }
 
   if (n_xatoms)
-    {
-#ifdef HAVE_XINTERNATOMS
-      XInternAtoms (GDK_DISPLAY_XDISPLAY (display),
-		    (char **)xatom_names, n_xatoms, False, xatoms);
-#else
-      for (i = 0; i < n_xatoms; i++)
-	xatoms[i] = XInternAtom (GDK_DISPLAY_XDISPLAY (display),
-				 xatom_names[i], False);
-#endif
-    }
+    XInternAtoms (GDK_DISPLAY_XDISPLAY (display),
+                  (char **)xatom_names, n_xatoms, False, xatoms);
 
   for (i = 0; i < n_xatoms; i++)
     insert_atom_pair (display, atoms[i], xatoms[i]);
