@@ -75,7 +75,8 @@ static const struct {
   {"Gtk/ShellShowsAppMenu",   "gtk-shell-shows-app-menu"},
   {"Gtk/ShellShowsMenubar",   "gtk-shell-shows-menubar"},
   {"Gtk/EnablePrimaryPaste",  "gtk-enable-primary-paste"},
-  {"Gtk/RecentFilesMaxAge",   "gtk-recent-files-max-age"}
+  {"Gtk/RecentFilesMaxAge",   "gtk-recent-files-max-age"},
+  {"Gtk/RecentFilesEnabled",  "gtk-recent-files-enabled"}
 };
 
 static const char *
