@@ -1,4 +1,4 @@
-/* GAIL - The GNOME Accessibility Enabling Library
+/* GTK+ - accessibility implementations
  * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,15 +21,24 @@
 #include <gtk/gtkpango.h>
 #include "gtklabelaccessible.h"
 
+struct _GtkLabelAccessiblePrivate
+{
+  gchar *text;
+  gint cursor_position;
+  gint selection_bound;
+};
 
 static void atk_text_interface_init (AtkTextIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (GtkLabelAccessible, _gtk_label_accessible, GTK_TYPE_WIDGET_ACCESSIBLE,
+G_DEFINE_TYPE_WITH_CODE (GtkLabelAccessible, gtk_label_accessible, GTK_TYPE_WIDGET_ACCESSIBLE,
                          G_IMPLEMENT_INTERFACE (ATK_TYPE_TEXT, atk_text_interface_init))
 
 static void
-_gtk_label_accessible_init (GtkLabelAccessible *label)
+gtk_label_accessible_init (GtkLabelAccessible *label)
 {
+  label->priv = G_TYPE_INSTANCE_GET_PRIVATE (label,
+                                             GTK_TYPE_LABEL_ACCESSIBLE,
+                                             GtkLabelAccessiblePrivate);
 }
 
 static void
@@ -39,13 +48,13 @@ gtk_label_accessible_initialize (AtkObject *obj,
   GtkWidget  *widget;
   GtkLabelAccessible *accessible;
 
-  ATK_OBJECT_CLASS (_gtk_label_accessible_parent_class)->initialize (obj, data);
+  ATK_OBJECT_CLASS (gtk_label_accessible_parent_class)->initialize (obj, data);
 
   accessible = GTK_LABEL_ACCESSIBLE (obj);
 
   widget = GTK_WIDGET (data);
 
-  accessible->text = g_strdup (gtk_label_get_text (GTK_LABEL (widget)));
+  accessible->priv->text = g_strdup (gtk_label_get_text (GTK_LABEL (widget)));
 
   /*
    * Check whether ancestor of GtkLabel is a GtkButton and if so
@@ -73,17 +82,17 @@ check_for_selection_change (GtkLabelAccessible *accessible,
 
   if (gtk_label_get_selection_bounds (label, &start, &end))
     {
-      if (end != accessible->cursor_position ||
-          start != accessible->selection_bound)
+      if (end != accessible->priv->cursor_position ||
+          start != accessible->priv->selection_bound)
         ret_val = TRUE;
     }
   else
     {
-      ret_val = (accessible->cursor_position != accessible->selection_bound);
+      ret_val = (accessible->priv->cursor_position != accessible->priv->selection_bound);
     }
 
-  accessible->cursor_position = end;
-  accessible->selection_bound = start;
+  accessible->priv->cursor_position = end;
+  accessible->priv->selection_bound = start;
 
   return ret_val;
 }
@@ -105,18 +114,18 @@ gtk_label_accessible_notify_gtk (GObject    *obj,
       const gchar *text;
 
       text = gtk_label_get_text (GTK_LABEL (widget));
-      if (g_strcmp0 (accessible->text, text) == 0)
+      if (g_strcmp0 (accessible->priv->text, text) == 0)
         return;
 
       /* Create a delete text and an insert text signal */
-      length = g_utf8_strlen (accessible->text, -1);
+      length = g_utf8_strlen (accessible->priv->text, -1);
       if (length > 0)
         g_signal_emit_by_name (atk_obj, "text-changed::delete", 0, length);
 
-      g_free (accessible->text);
-      accessible->text = g_strdup (text);
+      g_free (accessible->priv->text);
+      accessible->priv->text = g_strdup (text);
 
-      length = g_utf8_strlen (accessible->text, -1);
+      length = g_utf8_strlen (accessible->priv->text, -1);
       if (length > 0)
         g_signal_emit_by_name (atk_obj, "text-changed::insert", 0, length);
 
@@ -139,7 +148,7 @@ gtk_label_accessible_notify_gtk (GObject    *obj,
         g_signal_emit_by_name (atk_obj, "text-selection-changed");
     }
   else
-    GTK_WIDGET_ACCESSIBLE_CLASS (_gtk_label_accessible_parent_class)->notify_gtk (obj, pspec);
+    GTK_WIDGET_ACCESSIBLE_CLASS (gtk_label_accessible_parent_class)->notify_gtk (obj, pspec);
 }
 
 static void
@@ -147,9 +156,9 @@ gtk_label_accessible_finalize (GObject *object)
 {
   GtkLabelAccessible *accessible = GTK_LABEL_ACCESSIBLE (object);
 
-  g_free (accessible->text);
+  g_free (accessible->priv->text);
 
-  G_OBJECT_CLASS (_gtk_label_accessible_parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_label_accessible_parent_class)->finalize (object);
 }
 
 
@@ -165,7 +174,7 @@ gtk_label_accessible_ref_state_set (AtkObject *accessible)
   if (widget == NULL)
     return NULL;
 
-  state_set = ATK_OBJECT_CLASS (_gtk_label_accessible_parent_class)->ref_state_set (accessible);
+  state_set = ATK_OBJECT_CLASS (gtk_label_accessible_parent_class)->ref_state_set (accessible);
   atk_state_set_add_state (state_set, ATK_STATE_MULTI_LINE);
 
   return state_set;
@@ -183,7 +192,7 @@ gtk_label_accessible_ref_relation_set (AtkObject *obj)
   if (widget == NULL)
     return NULL;
 
-  relation_set = ATK_OBJECT_CLASS (_gtk_label_accessible_parent_class)->ref_relation_set (obj);
+  relation_set = ATK_OBJECT_CLASS (gtk_label_accessible_parent_class)->ref_relation_set (obj);
 
   if (!atk_relation_set_contains (relation_set, ATK_RELATION_LABEL_FOR))
     {
@@ -242,7 +251,7 @@ gtk_label_accessible_get_name (AtkObject *accessible)
 
   g_return_val_if_fail (GTK_IS_LABEL_ACCESSIBLE (accessible), NULL);
 
-  name = ATK_OBJECT_CLASS (_gtk_label_accessible_parent_class)->get_name (accessible);
+  name = ATK_OBJECT_CLASS (gtk_label_accessible_parent_class)->get_name (accessible);
   if (name != NULL)
     return name;
   else
@@ -263,7 +272,7 @@ gtk_label_accessible_get_name (AtkObject *accessible)
 }
 
 static void
-_gtk_label_accessible_class_init (GtkLabelAccessibleClass *klass)
+gtk_label_accessible_class_init (GtkLabelAccessibleClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
@@ -277,6 +286,8 @@ _gtk_label_accessible_class_init (GtkLabelAccessibleClass *klass)
   class->ref_state_set = gtk_label_accessible_ref_state_set;
   class->ref_relation_set = gtk_label_accessible_ref_relation_set;
   class->initialize = gtk_label_accessible_initialize;
+
+  g_type_class_add_private (klass, sizeof (GtkLabelAccessiblePrivate));
 }
 
 /* atktext.h */
