@@ -1,4 +1,4 @@
-/* GAIL - The GNOME Accessibility Implementation Library
+/* GTK+ - accessibility implementations
  * Copyright 2002 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,7 +25,7 @@ struct _GtkRadioMenuItemAccessiblePrivate
   GSList *old_group;
 };
 
-G_DEFINE_TYPE (GtkRadioMenuItemAccessible, _gtk_radio_menu_item_accessible, GTK_TYPE_CHECK_MENU_ITEM_ACCESSIBLE)
+G_DEFINE_TYPE (GtkRadioMenuItemAccessible, gtk_radio_menu_item_accessible, GTK_TYPE_CHECK_MENU_ITEM_ACCESSIBLE)
 
 
 static AtkRelationSet *
@@ -42,7 +42,7 @@ gtk_radio_menu_item_accessible_ref_relation_set (AtkObject *obj)
 
   radio_menu_item = GTK_RADIO_MENU_ITEM_ACCESSIBLE (obj);
 
-  relation_set = ATK_OBJECT_CLASS (_gtk_radio_menu_item_accessible_parent_class)->ref_relation_set (obj);
+  relation_set = ATK_OBJECT_CLASS (gtk_radio_menu_item_accessible_parent_class)->ref_relation_set (obj);
 
   /* If the radio menu_item's group has changed remove the relation */
   list = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (widget));
@@ -94,13 +94,13 @@ static void
 gtk_radio_menu_item_accessible_initialize (AtkObject *obj,
                                               gpointer   data)
 {
-  ATK_OBJECT_CLASS (_gtk_radio_menu_item_accessible_parent_class)->initialize (obj, data);
+  ATK_OBJECT_CLASS (gtk_radio_menu_item_accessible_parent_class)->initialize (obj, data);
 
   obj->role = ATK_ROLE_RADIO_MENU_ITEM;
 }
 
 static void
-_gtk_radio_menu_item_accessible_class_init (GtkRadioMenuItemAccessibleClass *klass)
+gtk_radio_menu_item_accessible_class_init (GtkRadioMenuItemAccessibleClass *klass)
 {
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
 
@@ -111,7 +111,7 @@ _gtk_radio_menu_item_accessible_class_init (GtkRadioMenuItemAccessibleClass *kla
 }
 
 static void
-_gtk_radio_menu_item_accessible_init (GtkRadioMenuItemAccessible *radio_menu_item)
+gtk_radio_menu_item_accessible_init (GtkRadioMenuItemAccessible *radio_menu_item)
 {
   radio_menu_item->priv = G_TYPE_INSTANCE_GET_PRIVATE (radio_menu_item,
                                                        GTK_TYPE_RADIO_MENU_ITEM_ACCESSIBLE,
